# locust error
not_found_locust_error: str = "locust not found"
# json error
cant_reformat_json_error: str = "can't reformat json is type right?"
wrong_json_data_error: str = "can't parser json"
cant_find_json_error: str = "can't find json"
cant_save_json_error: str = "can't save json"
# executor error
executor_data_error: str = "executor receive wrong data"
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# HTML
html_generate_no_data_tag: str = "record is None"
# add command
add_command_exception_tag: str = "command value type should be as method or function"
# argparse
argparse_get_wrong_data: str = "argparse receive wrong data"
